# Complete project details at https://RandomNerdTutorials.com/micropython-hc-sr04-ultrasonic-esp32-esp8266/
from hcsr04 import HCSR04
from time import sleep
from machine import Pin, ADC, PWM, I2C 
from time import sleep 
import time 
import network 
from config import ( 
    WIFI_SSID, WIFI_PASS, 
    MQTT_BROKER, MQTT_USER, MQTT_PASS, 
    TOPIC_PREFIX 
) 
from umqtt.simple import MQTTClient 
 
 
RED_GPIO = 42 
GREEN_GPIO = 40
# ESP32 sensor
sensor = HCSR04(trigger_pin=13, echo_pin=12, echo_timeout_us=10000)
TOPIC_SENSOR = f'{TOPIC_PREFIX}/sensor'
TOPIC_SIT_STATUS = 'b6610505551/sit'
 
 
def connect_wifi(): 
    mac = ':'.join(f'{b:02X}' for b in wifi.config('mac')) 
    print(f'WiFi MAC address is {mac}') 
    wifi.active(True) 
    print(f'Connecting to WiFi {WIFI_SSID}.') 
    wifi.connect(WIFI_SSID, WIFI_PASS) 
    while not wifi.isconnected(): 
        print('.', end='') 
        time.sleep(0.5) 
    print('\nWiFi connected.') 
 
 
 
 
def connect_mqtt(): 
    print(f'Connecting to MQTT broker at {MQTT_BROKER}.') 
    mqtt.connect() 
    mqtt.set_callback(mqtt_callback) 
    mqtt.subscribe(TOPIC_SIT_STATUS)
    print('MQTT broker connected.') 
 
 
 
 
def mqtt_callback(topic, payload):
    global active
    
    if topic.decode() == TOPIC_SIT_STATUS:
        active = payload.decode() == "True"
 
 
############ 
# setup 
############ 
red = Pin(RED_GPIO, Pin.OUT) 
green = Pin(GREEN_GPIO, Pin.OUT) 
 
wifi = network.WLAN(network.STA_IF) 
mqtt = MQTTClient(client_id='', 
                  server=MQTT_BROKER, 
                  user=MQTT_USER, 
                  password=MQTT_PASS)
connect_wifi() 
connect_mqtt() 
last_publish = 0 
red.value(0) 
green.value(0)

 
 
############ 
# loop 
############ 

active = False

while True:
    mqtt.check_msg()
    if active:
        distance = sensor.distance_cm()
        #checking distance between user's body and sensor
        if  2 < distance < 30:
            red.value(1)
            green.value(0)
            mqtt.publish(TOPIC_SENSOR, str(distance)) 
            last_publish = distance
            print(f'Distance: {distance:.2f} cm')
        if 30 < distance < 400:
            red.value(0)
            green.value(1)
            mqtt.publish(TOPIC_SENSOR, str(distance)) 
            last_publish = distance
            print(f'Distance: {distance:.2f} cm')
    sleep(0.5)
