from hcsr04 import HCSR04
from machine import Pin, PWM
import machine
from time import sleep
import time
import network
from config import (
    WIFI_SSID, WIFI_PASS,
    MQTT_BROKER, MQTT_USER, MQTT_PASS,
    TOPIC_PREFIX
)
from umqtt.simple import MQTTClient

TOPIC_SIT = f'{TOPIC_PREFIX}/sit'

wifi = network.WLAN(network.STA_IF)
mqtt = MQTTClient(client_id='',
                  server=MQTT_BROKER,
                  user=MQTT_USER,
                  password=MQTT_PASS)

def connect_wifi():
    mac = ':'.join(f'{b:02X}' for b in wifi.config('mac'))
    print(f'WiFi MAC address is {mac}')
    wifi.active(True)
    print(f'Connecting to WiFi {WIFI_SSID}.')
    wifi.connect(WIFI_SSID, WIFI_PASS)
    while not wifi.isconnected():
        print('.', end='')
        time.sleep(0.5)
    print('\nWiFi connected.')


def connect_mqtt():
    print(f'Connecting to MQTT broker at {MQTT_BROKER}.')
    mqtt.connect()
    mqtt.set_callback(mqtt_callback)
    print('MQTT broker connected.')

def mqtt_callback(topic, payload):
    pass

def play_tone(frequency, duration):
    pwm = PWM(buzzer_pin, freq=frequency, duty=512)  # ใช้ PWM เพื่อเล่นเสียง
    time.sleep_ms(duration)
    pwm.deinit()  # ปิด PWM
    
def sound_alert(alert):
    if alert:
        buzzer.duty(512)
    else:
        buzzer.duty(0)


# ESP32
connect_wifi()
connect_mqtt()
sensor = HCSR04(trigger_pin=13, echo_pin=12, echo_timeout_us=10000)
buzzer_pin = 10 
buzzer = machine.PWM(machine.Pin(buzzer_pin))
alert = False
sit = False
sound_alert(False)
start_time = time.time()
minimum_sit_distance_cm = 50
maximum_sit_time = 10
minimum_not_sit_time = 5

while True:
    mqtt.check_msg()
    distance = sensor.distance_cm()
    mqtt.publish(TOPIC_SIT, str(sit))
    print(f"{sit=}")
    print(f'Distance: {distance:.2f} cm -> ' + ('Yes' if distance < minimum_sit_distance_cm else 'No'))
    sleep(0.5)
    
    if not sit and distance < minimum_sit_distance_cm:
        sit = True
        if time.time() - start_time < minimum_not_sit_time:
            print("YOU ARE NOT ALLOWED TO SIT. PLEASE GET OUT.")
            alert = True
            sound_alert(True)
            continue 
        start_time = time.time()
    elif sit and distance < minimum_sit_distance_cm:
        if time.time() - start_time >= maximum_sit_time:
            print("YOU ARE OUT OF TIME. PLEASE GET THE OUT")
            alert = True
            sound_alert(True)
    elif sit and distance >= minimum_sit_distance_cm:
        sit =  False
        alert = False
        sound_alert(False)
        start_time = time.time()
        
            
